@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * bf_lib.c - non-GUI general functions
  *
- * Copyright (C) 2000-2006 Olivier Sessink
+ * Copyright (C) 2000-20010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* #define DEBUG */
+/*#define DEBUG */
 
 #include "config.h"
 
@@ -42,6 +41,10 @@
 #include "bluefish.h"  /* for DEBUG_MSG and stuff like that */
 #include "bf_lib.h"    /* myself */
 
+#ifdef DEVELOPMENT
+#include <stdlib.h>
+#endif
+
 /*#define REFP_REFS*/
 #ifdef REFP_REFS
 	gint num_refp_refs=0;
@@ -72,7 +75,7 @@ void refcpointer_unref(Trefcpointer *rp) {
 #ifdef REFP_DEBUG
 	g_print("refcpointer_unref, %p refcount=%d %s\n",rp, rp->count, (rp->count ==0 ? "freeing data" : ""));
 #endif
-	if (rp->count <= 0) {
+	if (G_UNLIKELY(rp->count <= 0)) {
 #ifdef REFP_REFS
 		num_refp_refs--;
 		g_print("num_refp_refs=%d\n",num_refp_refs);
@@ -295,13 +298,13 @@ void list_switch_order(GList *first, GList *second) {
 	second->data = tmp;
 }
 
-static gint length_common_prefix(gchar *first, gchar *second) {
+/*static gint length_common_prefix(gchar *first, gchar *second) {
 	gint i=0;
 	while (first[i] == second[i] && first[i] != '\0') {
 		i++;
 	}
 	return i;
-}
+}*/
 /**
  * find_common_prefix_in_stringlist:
  * @stringlist: a #GList* with strings
@@ -314,7 +317,7 @@ static gint length_common_prefix(gchar *first, gchar *second) {
  * 
  * Return value: #gint with number of common characters
  **/
-gint find_common_prefixlen_in_stringlist(GList *stringlist) {
+/*gint find_common_prefixlen_in_stringlist(GList *stringlist) {
 	gchar *firststring;
 	gint commonlen;
 	GList *tmplist;
@@ -332,7 +335,7 @@ gint find_common_prefixlen_in_stringlist(GList *stringlist) {
 		tmplist = g_list_next(tmplist);
 	}
 	return commonlen;
-}
+}*/
 /**
  * countchars:
  * @string: a gchar * to count the chars in
@@ -607,25 +610,26 @@ large php files, a cache of
 so we keep it at 12 for the moment
 */
 #define UTF8_OFFSET_CACHE_SIZE 12
-/* #define UTF8_BYTECHARDEBUG */
+/*#define UTF8_BYTECHARDEBUG */
 
 typedef struct {
-#ifdef DEVELOPMENT
-	gchar *last_buf;
-#endif
 	/* the two arrays must be grouped and in this order, because they are moved back 
 	one position in ONE memmove() call */
-	guint  last_byteoffset[UTF8_OFFSET_CACHE_SIZE];
-	guint  last_charoffset[UTF8_OFFSET_CACHE_SIZE];
+	guint last_byteoffset[UTF8_OFFSET_CACHE_SIZE];
+	guint last_charoffset[UTF8_OFFSET_CACHE_SIZE];
 #ifdef UTF8_BYTECHARDEBUG
 	guint numcalls_since_reset;
 	unsigned long long int numbytes_parsed;
 	guint numcalls_cached_since_reset;
 	unsigned long long int numbytes_cached_parsed;
+	guint num_memmoves;
+#endif
+#ifdef DEVELOPMENT
+	gchar *last_buf;
 #endif
 } Tutf8_offset_cache;
 
-static Tutf8_offset_cache utf8_offset_cache;
+static Tutf8_offset_cache utf8_offset_cache = {{0,0,0,0,0,0,0,0,0,0,0,0}, {0,0,0,0,0,0,0,0,0,0,0,0}};
 
 /**
  * utf8_offset_cache_reset:
@@ -640,11 +644,52 @@ __inline__
 #endif
 void utf8_offset_cache_reset() {
 #ifdef UTF8_BYTECHARDEBUG
-	g_print("UTF8_BYTECHARDEBUG: called %d times for total %llu bytes\n",utf8_offset_cache.numcalls_since_reset,utf8_offset_cache.numbytes_parsed);
-	g_print("UTF8_BYTECHARDEBUG: cache HIT %d times, reduced to %llu bytes, cache size %d\n",utf8_offset_cache.numcalls_cached_since_reset,utf8_offset_cache.numbytes_cached_parsed,UTF8_OFFSET_CACHE_SIZE);
+	g_print("UTF8_BYTECHARDEBUG: called %d times for total %llu bytes\n"
+				,utf8_offset_cache.numcalls_since_reset
+				,utf8_offset_cache.numbytes_parsed);
+	g_print("UTF8_BYTECHARDEBUG: cache HIT %d times, reduced from %llu to %llu bytes (%4.2f%%), cache size %d, num memmoves=%d\n"
+				,utf8_offset_cache.numcalls_cached_since_reset
+				,utf8_offset_cache.numbytes_parsed
+				,utf8_offset_cache.numbytes_cached_parsed
+				,((gfloat)utf8_offset_cache.numbytes_cached_parsed/(gfloat)utf8_offset_cache.numbytes_parsed*100.0)
+				,UTF8_OFFSET_CACHE_SIZE
+				,utf8_offset_cache.num_memmoves);
 #endif
 	memset(&utf8_offset_cache, 0, sizeof(Tutf8_offset_cache));
 }
+
+guint utf8_charoffset_to_byteoffset_cached(const gchar *string, guint charoffset) {
+	guint i=UTF8_OFFSET_CACHE_SIZE-1, retval;
+	
+	if (charoffset ==0) return 0;
+	/*g_print("requested charoffset %d\n",charoffset);*/
+	while (i > 0 && utf8_offset_cache.last_charoffset[i] > charoffset) {
+		i--;
+	}
+	/*g_print("cache pos %d has charoffset %d and byteoffset %d\n",i,utf8_offset_cache.last_charoffset[i],utf8_offset_cache.last_byteoffset[i]);*/
+	if (i > 0) {
+		const gchar *tmp;
+		if (utf8_offset_cache.last_charoffset[i] == charoffset) {
+			return utf8_offset_cache.last_byteoffset[i];
+		}
+		tmp = string+utf8_offset_cache.last_byteoffset[i];
+		retval = (g_utf8_offset_to_pointer(tmp, charoffset-utf8_offset_cache.last_charoffset[i])-tmp)+utf8_offset_cache.last_byteoffset[i];
+		/*g_print("non-cached byteoffset %d, cached byteoffset %d for charoffset %d\n",(g_utf8_offset_to_pointer(string, charoffset)-string),retval,charoffset);*/
+	} else {
+		retval = (g_utf8_offset_to_pointer(string, charoffset)-string);
+		/*g_print("non-cached byteoffset %d for charoffset %d\n",retval,charoffset);*/
+	}
+	if (i == (UTF8_OFFSET_CACHE_SIZE-1)) {
+		/* add this new calculation to the cache */
+		/* this is a nasty trick to move all guint entries one back in the array, so we can add the new one */
+		memmove(&utf8_offset_cache.last_byteoffset[0], &utf8_offset_cache.last_byteoffset[1], (UTF8_OFFSET_CACHE_SIZE+UTF8_OFFSET_CACHE_SIZE-1)*sizeof(guint));
+		/*g_print("add byteoffset %d charoffset %d to position %d in the cache\n",retval,charoffset,UTF8_OFFSET_CACHE_SIZE-1);*/
+		utf8_offset_cache.last_byteoffset[UTF8_OFFSET_CACHE_SIZE-1] = retval;
+		utf8_offset_cache.last_charoffset[UTF8_OFFSET_CACHE_SIZE-1] = charoffset;
+	}
+	return retval;
+}
+
 /**
  * utf8_byteoffset_to_charsoffset_cached:
  * @string: the gchar * you want to count
@@ -688,7 +733,7 @@ guint utf8_byteoffset_to_charsoffset_cached(const gchar *string, glong byteoffse
 		we are not supposed to get called in the middle of a character)*/
 		retval = g_utf8_pointer_to_offset(string+utf8_offset_cache.last_byteoffset[i], string+byteoffset)+utf8_offset_cache.last_charoffset[i];
 #ifdef UTF8_BYTECHARDEBUG
-		utf8_offset_cache.numbytes_parsed += (byteoffset - utf8_offset_cache.last_byteoffset[i]);
+		utf8_offset_cache.numbytes_parsed += byteoffset;
 		utf8_offset_cache.numbytes_cached_parsed += (byteoffset - utf8_offset_cache.last_byteoffset[i]);
 		utf8_offset_cache.numcalls_cached_since_reset++;
 #endif
@@ -701,6 +746,9 @@ guint utf8_byteoffset_to_charsoffset_cached(const gchar *string, glong byteoffse
 	DEBUG_MSG(" and byteoffset %ld has charoffset %d\n",byteoffset,retval);
 	if (i == (UTF8_OFFSET_CACHE_SIZE-1)) {
 		/* add this new calculation to the cache */
+#ifdef UTF8_BYTECHARDEBUG
+		utf8_offset_cache.num_memmoves++;
+#endif
 		/* this is a nasty trick to move all guint entries one back in the array, so we can add the new one */
 		memmove(&utf8_offset_cache.last_byteoffset[0], &utf8_offset_cache.last_byteoffset[1], (UTF8_OFFSET_CACHE_SIZE+UTF8_OFFSET_CACHE_SIZE-1)*sizeof(guint));
 
@@ -713,115 +761,6 @@ guint utf8_byteoffset_to_charsoffset_cached(const gchar *string, glong byteoffse
 	return retval;
 }
 
-/**
- * escapestring:
- * @original: a gchar * to escape
- * @delimiter: a gchar that needs escaping, use '\0' if you don't need one
- *
- * this function will backslash escape \n, \t, and \ characters, and if 
- * there is a delimiter it will also be escaped
- * 
- * Return value: a newly allocated gchar * that is escaped
- **/
-/*gchar *old_escapestring(gchar *original, gchar delimiter)
-{
-	gchar *tmp, *newstring, *escapedchars;
-	guint newsize, pos=0;
-
-	* count the size of the new string *
-	escapedchars = g_strdup_printf("\n\t\\%c", delimiter);
-	DEBUG_MSG("escapestring, escapedchars=%s, extra length=%d\n", escapedchars, countchars(original, escapedchars));
-	newsize = countchars(original, escapedchars) + strlen(original) + 1;
-	newstring = g_malloc0(newsize * sizeof(gchar));
-	g_free(escapedchars);
-	DEBUG_MSG("escapestring, original=%s, newsize=%d\n", original, newsize);
-
-	tmp = original;
-	while (*tmp != '\0') {
-		switch (*tmp) {
-		case '\\':
-			strcat(newstring, "\\\\");
-			pos +=2;
-		break;
-		case '\n':
-			strcat(newstring, "\\n");
-			pos +=2;
-		break;
-		case '\t':
-			strcat(newstring, "\\t");
-			pos +=2;
-		break;
-		default:
-			if (*tmp == delimiter) {
-				newstring[pos] = '\\';
-				newstring[pos+1] = delimiter;
-				pos +=2;			
-			} else {
-				newstring[pos] = *tmp;
-				pos++;
-			}
-		break;
-		}
-		newstring[pos] = '\0';
-		tmp++;
-	}
-	DEBUG_MSG("escapestring, newstring = %s\n", newstring);
-	return newstring;
-}*/
-
-/**
- * unescapestring:
- * @original: a gchar * to unescape
- *
- * this function will backslash unescape \n, \t, and \\ sequences to 
- * their characters, and if there is any other escaped character 
- * it will be replaced without the backslash
- * 
- * Return value: a newly allocated gchar * that is unescaped
- **/
-/*gchar *old_unescapestring(gchar *original)
-{
-	gchar *tmp1, *tmp2, *newstring;
-	guint newsize;
-	gint escaped;
-
-	newsize = strlen(original) + 1;
-	newstring = g_malloc0(newsize * sizeof(gchar));
-	DEBUG_MSG("unescapestring, original=%s, newsize = %d\n", original, newsize);
-
-	tmp1 = original;
-	tmp2 = newstring;
-	escaped = 0;
-	while (*tmp1 != '\0') {
-		if (escaped) {
-			switch (*tmp1) {
-			case '\\':
-				*tmp2++ = '\\';
-			break;
-			case 'n':
-				*tmp2++ = '\n';
-			break;
-			case 't':
-				*tmp2++ = '\t';
-			break;
-			default:
-				*tmp2++ = *tmp1;
-			break;
-			}
-
-			escaped = 0;
-		} else {
-			if (*tmp1 == '\\')
-				escaped = 1;
-			else
-				*tmp2++ = *tmp1;
-		}
-		tmp1++;
-	}
-	DEBUG_MSG("unescapestring, newstring = %s\n", newstring);
-	return newstring;
-}*/
-
 /**
  * strip_any_whitespace:
  * @string: a gchar * to strip
@@ -832,6 +771,7 @@ guint utf8_byteoffset_to_charsoffset_cached(const gchar *string, glong byteoffse
  * 
  * Return value: the same gchar * as passed to the function
  **/
+/* used only in the htmlbar plugin */
 gchar *strip_any_whitespace(gchar *string) {
 	gint count=0, len;
 
@@ -980,9 +920,10 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	eff_link_to_filepath = most_efficient_filename(g_strdup(link_to_filepath));
 	DEBUG_MSG("eff_current: '%s'\n",eff_current_filepath);
 	DEBUG_MSG("eff_link_to: '%s'\n",eff_link_to_filepath);
-	/* get the size of the directory of the current_filename */
-	current_filename_length = strlen(strrchr(eff_current_filepath, DIRCHR))-1;
-	link_to_filename_length = strlen(strrchr(eff_link_to_filepath, DIRCHR))-1;
+	/* get the size of the directory of the current_filename. we work on URI's
+	so we always use the / as character */
+	current_filename_length = strlen(strrchr(eff_current_filepath, '/'))-1;
+	link_to_filename_length = strlen(strrchr(eff_link_to_filepath, '/'))-1;
 	DEBUG_MSG("create_relative_link_to, filenames: current: %d, link_to:%d\n", current_filename_length,link_to_filename_length); 
 	current_dirname_length = strlen(eff_current_filepath) - current_filename_length;
 	link_to_dirname_length = strlen(eff_link_to_filepath) - link_to_filename_length;
@@ -1008,7 +949,7 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	/* this is the common length, but we need the common directories */
 	if (eff_current_filepath[common_lenght] != DIRCHR) {
 		gchar *ltmp = &eff_current_filepath[common_lenght];
-		while ((*ltmp != DIRCHR) && (common_lenght > 0)) {
+		while ((*ltmp != '/') && (common_lenght > 0)) {
 			common_lenght--;
 			ltmp--;
 		}
@@ -1019,7 +960,7 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	   is compared to the link_to_file, that is the amount of ../ we need to add */
 	deeper_dirs = 0;
 	for (count = common_lenght+1; count <= current_dirname_length; count++) {
-		if (eff_current_filepath[count] == DIRCHR) {
+		if (eff_current_filepath[count] == '/') {
 			deeper_dirs++;
 			DEBUG_MSG("create_relative_link_to, on count=%d, deeper_dirs=%d\n", count, deeper_dirs);
 		}
@@ -1085,14 +1026,14 @@ gchar *ending_slash(const gchar *dirname) {
  *
  * Return value: a newly allocated gchar * dirname that does end on a '/', or NULL on failure
  **/
-gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
+/*gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
 	gchar *tmp = strrchr(filename, DIRCHR);
 	if (tmp) {
 		return g_strndup(filename, (tmp - filename + 1));
 	} else {
 		return NULL;
 	}
-}
+}*/
 
 /**
  * full_path_exists:
@@ -1147,7 +1088,7 @@ GFile *return_first_existing_filename(const gchar *filename, ...) {
  *
  * Return value: gboolean, TRUE if the file has one of the extensions in the array
  **/
-gboolean filename_test_extensions(gchar **extensions, const gchar *filename) {
+/*gboolean filename_test_extensions(gchar **extensions, const gchar *filename) {
 	gint fnlen;
 	if (!extensions) {
 		return FALSE;
@@ -1162,7 +1103,7 @@ gboolean filename_test_extensions(gchar **extensions, const gchar *filename) {
 		extensions++;
 	}
 	return FALSE;
-}
+}*/
 
 /**
  * bf_str_repeat:
@@ -1482,10 +1423,10 @@ gchar *gfile_display_name(GFile *uri, GFileInfo *finfo) {
 
 gboolean gfile_uri_is_parent(GFile *parent, GFile *child, gboolean recursive) {
 	if (parent == NULL) {
-		g_print("gfile_uri_is_parent, parent=NULL ??\n");
+		DEBUG_MSG("gfile_uri_is_parent, parent=NULL ??\n");
 		return FALSE;
 	} else if (child == NULL) {
-		g_print("gfile_uri_is_parent, child=NULL ??\n");
+		DEBUG_MSG("gfile_uri_is_parent, child=NULL ??\n");
 		return FALSE;
 	} else {
 		gboolean retval = FALSE;
@@ -1521,3 +1462,53 @@ gchar *get_hostname_from_uri(GFile *uri) {
 	return retval;
 }
 
+void
+callback_register(GSList **slist, void (*func)(), gpointer data) {
+	Tcallback *cb;
+	cb = g_slice_new0(Tcallback);
+	cb->func = func;
+	cb->data = data;
+	*slist = g_slist_append(*slist, cb);	
+}
+
+void
+callback_remove_by_data(GSList **slist, gpointer data) {
+	Tcallback *cb=NULL;
+	GSList *tmpslist=*slist;
+	while(tmpslist) {
+		cb=tmpslist->data;
+		if (cb->data == data) {
+			*slist = g_slist_delete_link(*slist, tmpslist);
+			g_slice_free(Tcallback, cb);
+			return;
+		}
+		tmpslist = g_slist_next(tmpslist);
+	}
+}
+
+void
+callback_remove_all(GSList **slist) {
+	GSList *tmpslist=*slist;
+	while(tmpslist) {
+		g_slice_free(Tcallback, tmpslist->data);
+		tmpslist = g_slist_next(tmpslist);
+	}
+	g_slist_free(*slist);
+	*slist=NULL;
+}
+
+#ifndef HAVE_STRCASESTR
+#include <ctype.h>
+char *strcasestr(char *a, char *b)
+{
+	size_t l;
+	char f[3];
+
+	snprintf(f, sizeof(f), "%c%c", tolower(*b), toupper(*b));
+	for (l = strcspn(a, f); l != strlen(a); l += strcspn(a + l + 1, f) + 1)
+		if (strncasecmp(a + l, b, strlen(b)) == 0)
+			return(a + l);
+	return(NULL);
+}
+#endif
+
