@@ -178,6 +178,7 @@ static gboolean startup_in_idle(gpointer data) {
 			fb2config_init();			/* filebrowser2config */
 			filters_rebuild();
 			main_v->bmarkdata = bookmark_data_new();
+			register_bf_stock_icons();
 			/* create the first window */
 			startup->firstbfwin = g_new0(Tbfwin, 1);
 			startup->firstbfwin->session = main_v->session;
@@ -292,15 +293,6 @@ int main(int argc, char *argv[])
 	textdomain(PACKAGE);
 #endif							/* ENABLE_NLS */
 
-	gtk_rc_parse_string ("style \"bluefish-small-close-button-style\"\n"
-                       "{\n"
-                          "GtkWidget::focus-padding = 0\n"
-                          "GtkWidget::focus-line-width = 0\n"
-                          "xthickness = 0\n"
-                          "ythickness = 0\n"
-                       "}\n"
-                       "widget \"*.bluefish-small-close-button\" style \"bluefish-small-close-button-style\""
-                       );
 	context = g_option_context_new(_(" [FILE(S)]"));
 #ifdef ENABLE_NLS
 	g_option_context_add_main_entries(context, options, PACKAGE);
@@ -320,6 +312,18 @@ int main(int argc, char *argv[])
 	if (error)
 		g_error_free(error);
 
+#if !GTK_CHECK_VERSION(3,0,0)
+	gtk_rc_parse_string ("style \"bluefish-small-close-button-style\"\n"
+                       "{\n"
+                          "GtkWidget::focus-padding = 0\n"
+                          "GtkWidget::focus-line-width = 0\n"
+                          "xthickness = 0\n"
+                          "ythickness = 0\n"
+                       "}\n"
+                       "widget \"*.bluefish-small-close-button\" style \"bluefish-small-close-button-style\""
+                       );
+#endif
+
 	xmlInitParser();
 	startup = g_new0(Tstartup,1);
 	main_v = g_new0(Tmain, 1);
@@ -413,6 +417,7 @@ int main(int argc, char *argv[])
 	DEBUG_MSG("calling fb2config_cleanup()\n");
 	fb2config_cleanup();
 	main_v->bmarkdata = bookmark_data_cleanup(main_v->bmarkdata);
+	bluefish_cleanup_plugins();
 	langmgr_cleanup();
 	xmlCleanupParser();
 	
