@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * external_commands.c - backend for external commands, filters and the outputbox
  *
- * Copyright (C) 2005-2008 Olivier Sessink
+ * Copyright (C) 2005-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* #define DEBUG */
+/*#define DEBUG*/
 
 #include <gtk/gtk.h>
 #include <string.h>
@@ -44,36 +43,43 @@
  *
  */
 typedef struct {
-	gint refcount;
+	gint refcount; /* the reference count for this structure, on 0 we can free this structure */
 	Tbfwin *bfwin;
 	Tdocument *doc;
 	gint begin; /* 0 for begin of the file */
 	gint end; /* -1 for end of the file */
-	const gchar *formatstring;
-	gchar *commandstring;
+	const gchar *formatstring; /* the command from the configuration, so including placeholders such as %i, %f, |,  etc. */
+	gchar *commandstring; /* the command that will be started by g_spawn, the placeholders should have been replaced/removed */
 
 	gchar *securedir; /* if we need any local temporary files for input or output we'll use fifo's */
-	gchar *fifo_in;
+	gchar *fifo_in; /* the path for the iput and output fifo */
 	gchar *fifo_out;
-	gchar *tmp_in;
+	gchar *tmp_in; /* the path for the input/output temporary filename */
 	gchar *tmp_out;
-	gchar *inplace;
-	gboolean pipe_in;
+	gchar *inplace; /* if we use in-place editing, so the input file will become the output file */
+	gboolean pipe_in; /* if we use pipes for input and output */
 	gboolean pipe_out;
+
+#ifdef WIN32
+	gpointer fifo_in_handle; /* the path for 'named pipes' on win32 */
+	gpointer fifo_out_handle;
+#endif
+
+	gboolean include_stderr; /* if stderr should be included in the output */
 	
-	gboolean include_stderr;
+	gint standard_input; /* the pipes that will be filled by g_spawn_async_with_pipes */
+	gint standard_output; 
 	
-	GIOChannel* channel_in;
-	gchar *buffer_out;
-	gchar *buffer_out_position;
-	GIOChannel* channel_out;
+	GIOChannel* channel_in; /* the GIO channels for input into the filter */
+	gchar *buffer_out; /* the buffer that is the input for the filter */
+	gchar *buffer_out_position; /* a pointer inside the above buffer that points to the data that has not yet been written to the filter */
 	
-	GIOFunc channel_out_lcb;
-	gpointer channel_out_data;
+	GIOChannel* channel_out; /* the GIO channel for output from the filter */
+	GIOFunc channel_out_lcb; /* the callback that will be registered for output data  */
+	gpointer channel_out_data; /* the corresponding callback data */
 	
-	GPid child_pid;
-	
-	gpointer data;
+	GPid child_pid; /* PID of the filter */
+	guint start_command_idle_id;
 } Texternalp;
 
 static void start_command_backend(Texternalp *ep);
@@ -85,11 +91,21 @@ static void externalp_unref(Texternalp *ep) {
 		DEBUG_MSG("externalp_unref, cleanup!\n");
 		if (ep->commandstring) g_free(ep->commandstring);
 		if (ep->fifo_in) {
-			unlink(ep->fifo_in);
+			#ifdef WIN32
+				CloseHandle(ep->fifo_in_handle);
+				g_free(ep->fifo_in_handle);
+			#else
+				unlink(ep->fifo_in);
+			#endif
 			g_free(ep->fifo_in);
 		}
 		if (ep->fifo_out) {
-			unlink(ep->fifo_out);
+			#ifdef WIN32
+				CloseHandle(ep->fifo_out_handle);
+				g_free(ep->fifo_out_handle);
+			#else
+				unlink(ep->fifo_out);
+			#endif
 			g_free(ep->fifo_out);
 		}
 		if (ep->tmp_in) {
@@ -110,6 +126,10 @@ static void externalp_unref(Texternalp *ep) {
 			rmdir(ep->securedir);
 			g_free(ep->securedir);
 		}
+		if (ep->channel_in)
+			g_io_channel_unref(ep->channel_in);
+		if (ep->channel_out)
+			g_io_channel_unref(ep->channel_out);
 		g_free(ep);
 	}
 }
@@ -162,6 +182,15 @@ static void child_watch_lcb(GPid pid,gint status,gpointer data) {
 	Texternalp *ep = data;
 	GError *gerror=NULL;
 	DEBUG_MSG("child_watch_lcb, child exited with status=%d\n",status);
+	
+	if (!ep->buffer_out) {
+		/* the child has exited before we actually started to write data to the child, just abort now */
+		g_source_remove(ep->start_command_idle_id);
+		externalp_unref(ep); /* unref twice, once for the start_command_idle, once for child_watch_lcb */
+		externalp_unref(ep);
+		return;
+	}
+	
 	/* if there was a temporary output file, we should now open it and start to read it */
 	if (ep->tmp_out) {
 		
@@ -189,17 +218,68 @@ static void child_watch_lcb(GPid pid,gint status,gpointer data) {
 	externalp_unref(ep);
 }
 
-#ifndef USEBINSH
-static gint count_char(char *string, char mychar) {
-	gint retval = 0;
-	gchar *tmp = string;
-	while (*tmp) {
-		if (*tmp == mychar)
-			retval++;
-	}
-	return retval;
-}
+static gboolean
+start_command_idle(gpointer data)
+{
+	Texternalp *ep = data;
+	GError *gerror=NULL;
+
+	if (ep->pipe_in) {
+		DEBUG_MSG("start_command_idle, creating channel_in from pipe\n");
+#ifdef WIN32
+		ep->channel_in = g_io_channel_win32_new_fd(ep->standard_input);
+#else
+		ep->channel_in = g_io_channel_unix_new(ep->standard_input);
 #endif
+	} else if (ep->fifo_in) {
+		DEBUG_MSG("start_command_idle, connecting channel_in to fifo %s\n",ep->fifo_in);
+		/* problem: this can hang if there is no process actually reading from this fifo... so if the 
+		command died in some way (a nonexisting command), this call will hang bluefish */
+		ep->channel_in = g_io_channel_new_file(ep->fifo_in,"w",&gerror);
+		if (gerror) {
+			DEBUG_MSG("start_command_idle, error connecting to fifo %s: %s\n",ep->fifo_in,gerror->message);
+			g_warning("failed to create fifo %s %d: %s\n",ep->fifo_in, gerror->code, gerror->message);
+			g_error_free(gerror);
+			/* BUG TODO: free Texternalp */
+			return FALSE;
+		}
+	}
+	if (ep->pipe_in || ep->fifo_in) {
+		ep->refcount++;
+		ep->buffer_out = ep->buffer_out_position = doc_get_chars(ep->bfwin->current_document,ep->begin,ep->end);
+		g_io_channel_set_flags(ep->channel_in,G_IO_FLAG_NONBLOCK,NULL);
+		/* now we should start writing, correct ? */
+		DEBUG_MSG("start_command_idle, begin=%d, end=%d, add watch for channel_in\n",ep->begin, ep->end);
+		g_io_add_watch(ep->channel_in,G_IO_OUT,start_command_write_lcb,ep);
+	}
+	if (ep->pipe_out) {
+#ifdef WIN32
+		ep->channel_out = g_io_channel_win32_new_fd(ep->standard_output);
+#else
+		ep->channel_out = g_io_channel_unix_new(ep->standard_output);
+#endif
+		DEBUG_MSG("start_command_idle, created channel_out from pipe\n");
+	} else if (ep->fifo_out) {
+		ep->channel_out = g_io_channel_new_file(ep->fifo_out,"r",&gerror);
+		DEBUG_MSG("start_command_idle, created channel_out from fifo %s\n",ep->fifo_out);
+		if (gerror) {
+			g_warning("failed to start reading from file %s %d: %s\n",ep->fifo_out, gerror->code, gerror->message);
+			g_error_free(gerror);
+			/* BUG TODO: free Texternalp */
+			return FALSE;
+		}
+	}
+	if (ep->channel_out_lcb && (ep->pipe_out || ep->fifo_out)) {
+		ep->refcount++;
+		g_io_channel_set_flags(ep->channel_out,G_IO_FLAG_NONBLOCK,NULL);
+		DEBUG_MSG("start_command_idle, add watch for channel_out\n");
+		g_io_add_watch(ep->channel_out, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP,ep->channel_out_lcb,ep->channel_out_data);
+	}
+	externalp_unref(ep);
+	
+	
+	return FALSE; /* don't call me again */
+}
 
 static void start_command_backend(Texternalp *ep) {
 #ifdef USEBINSH	
@@ -207,43 +287,59 @@ static void start_command_backend(Texternalp *ep) {
 #else
 	gchar **argv;
 #endif
-	gint standard_input=0,standard_output=0;
 	GError *error=NULL;
 
+	if (!ep->bfwin->current_document)
+		return;
+
 #ifdef USEBINSH
 	argv[0] = "/bin/sh";
 	argv[1] = "-c";
 	argv[2] = ep->commandstring;
 	argv[3] = NULL;
 #else
-	argv = g_malloc(sizeof(char *)*(count_char(ep->commandstring, ' ')+1));
+    g_shell_parse_argv(ep->commandstring, NULL, &argv, NULL);
 #endif
 	DEBUG_MSG("start_command_backend,commandstring=%s\n",ep->commandstring);
-#ifndef WIN32
+#ifdef WIN32
+	ep->include_stderr = FALSE;
+	if (ep->fifo_in) {
+		ep->fifo_in = g_strdup("\\\\.\\pipe\\bluefish_in");
+		if ((ep->fifo_in_handle = CreateNamedPipe(ep->fifo_in, PIPE_ACCESS_INBOUND, PIPE_TYPE_BYTE | PIPE_NOWAIT, 
+			 PIPE_UNLIMITED_INSTANCES, 2048, 2048, 100, NULL)) == INVALID_HANDLE_VALUE) {
+			g_print("some error happened creating fifo %s??\n", ep->fifo_in);
+			return;
+		}
+	}
+	if (ep->fifo_out) {
+		ep->fifo_out = g_strdup("\\\\.\\pipe\\bluefish_out");
+		if ((ep->fifo_out_handle = CreateNamedPipe(ep->fifo_out, PIPE_ACCESS_OUTBOUND, PIPE_TYPE_BYTE | PIPE_NOWAIT, 
+			 PIPE_UNLIMITED_INSTANCES, 2048, 2048, 100, NULL)) == INVALID_HANDLE_VALUE) {
+			g_print("some error happened creating fifo %s??\n", ep->fifo_out);
+			return;
+		}
+	}
+#else
 	if (ep->fifo_in) {
 		if (mkfifo(ep->fifo_in, 0600) != 0) {
-			g_print("some error happened creating fifo %s??\n",ep->fifo_in);
+			DEBUG_MSG("some error happened creating fifo %s??\n",ep->fifo_in);
 			return;
 		}
 		DEBUG_MSG("start_command_backend, created fifo %s\n",ep->fifo_in);
 	}
 	if (ep->fifo_out) {
 		if (mkfifo(ep->fifo_out, 0600) != 0) {
-			g_print("some error happened creating fifo %s??\n",ep->fifo_out);
+			DEBUG_MSG("some error happened creating fifo %s??\n",ep->fifo_out);
 			return;
 		}
 	}
-#else
-	DEBUG_MSG("start_command_backend,commandstring=%s\n",ep->commandstring);
-	DEBUG_MSG("unsupported function mkfifo\n");
-	ep->include_stderr = FALSE;
 #endif
 	DEBUG_MSG("start_command_backend, pipe_in=%d, pipe_out=%d, fifo_in=%s, fifo_out=%s,include_stderr=%d\n",ep->pipe_in,ep->pipe_out,ep->fifo_in,ep->fifo_out,ep->include_stderr);
 	DEBUG_MSG("start_command_backend, about to spawn process /bin/sh -c %s\n",argv[2]);
-	g_spawn_async_with_pipes(NULL,argv,NULL,G_SPAWN_DO_NOT_REAP_CHILD,(ep->include_stderr)?spawn_setup_lcb:NULL,ep,&ep->child_pid,
-				(ep->pipe_in) ? &standard_input : NULL,
-				(ep->pipe_out) ? &standard_output : NULL,
-				NULL,&error);
+	g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, (ep->include_stderr)?spawn_setup_lcb:NULL, ep, &ep->child_pid, 
+				(ep->pipe_in) ? &ep->standard_input : NULL,
+				(ep->pipe_out) ? &ep->standard_output : NULL,
+				NULL, &error);
 	ep->refcount+=2; /* one reference for this function (cleared in the end), one reference for the child_watch function */
 	g_child_watch_add(ep->child_pid,child_watch_lcb,ep);
 	if (error) {
@@ -253,52 +349,13 @@ static void start_command_backend(Texternalp *ep) {
 		externalp_unref(ep);
 		return;
 	}
-	flush_queue();
-	if (ep->pipe_in) {
-		DEBUG_MSG("start_command_backend, creating channel_in from pipe\n");
-		ep->channel_in = g_io_channel_unix_new(standard_input);
-	} else if (ep->fifo_in) {
-		DEBUG_MSG("start_command_backend, connecting channel_in to fifo %s\n",ep->fifo_in);
-		/* problem: this can hang if there is no process actually reading from this fifo... so if the 
-		command died in some way (a nonexisting command), this call will hang bluefish */
-		ep->channel_in = g_io_channel_new_file(ep->fifo_in,"w",&error);
-		if (error) {
-			DEBUG_MSG("start_command_backend, error connecting to fifo %s: %s\n",ep->fifo_in,error->message);
-			g_warning("failed to create fifo %s %d: %s\n",ep->fifo_in, error->code, error->message);
-			g_error_free(error);
-			return;
-		}
-	}
-	if (ep->pipe_in || ep->fifo_in) {
-		ep->refcount++;
-		ep->buffer_out = ep->buffer_out_position = doc_get_chars(ep->bfwin->current_document,ep->begin,ep->end);
-		g_io_channel_set_flags(ep->channel_in,G_IO_FLAG_NONBLOCK,NULL);
-		/* now we should start writing, correct ? */
-		DEBUG_MSG("start_command_backend, begin=%d, end=%d, add watch for channel_in\n",ep->begin, ep->end);
-		g_io_add_watch(ep->channel_in,G_IO_OUT,start_command_write_lcb,ep);
-	}
-	if (ep->pipe_out) {
-		ep->channel_out = g_io_channel_unix_new(standard_output);
-		DEBUG_MSG("start_command_backend, created channel_out from pipe\n");
-	} else if (ep->fifo_out) {
-		ep->channel_out = g_io_channel_new_file(ep->fifo_out,"r",&error);
-		DEBUG_MSG("start_command_backend, created channel_out from fifo %s\n",ep->fifo_out);
-		if (error) {
-			g_warning("failed to start reading from file %s %d: %s\n",ep->fifo_out, error->code, error->message);
-			g_error_free(error);
-			return;
-		}
-	}
-	if (ep->channel_out_lcb && (ep->pipe_out || ep->fifo_out)) {
-		ep->refcount++;
-		g_io_channel_set_flags(ep->channel_out,G_IO_FLAG_NONBLOCK,NULL);
-		DEBUG_MSG("start_command_backend, add watch for channel_out\n");
-		g_io_add_watch(ep->channel_out, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP,ep->channel_out_lcb,ep->channel_out_data);
-	}
-	externalp_unref(ep);
+	ep->start_command_idle_id = g_idle_add(start_command_idle, ep);
 }
 
 static void start_command(Texternalp *ep) {
+	if (!ep->bfwin->current_document)
+		return;
+	
 	if (ep->tmp_in) {
 		GError *gerror=NULL;
 		/* first create tmp_in, then start the real command in the callback */
@@ -358,7 +415,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	if (!ep->bfwin->current_document) {
 		return NULL;
 	}
-	
+
 	formatstring = g_strstrip(g_strdup(formatstr));
 	formatstringlen = strlen(formatstring);
 
@@ -448,7 +505,11 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		curi = g_file_get_uri(ep->bfwin->current_document->uri);
 
 		if (need_local || (is_local_non_modified && (need_tmpin||need_fifoin))) {
-			localfilename = strrchr(localname, '/')+1;
+			#ifdef WIN32
+				localfilename = strrchr(localname, '\\')+1;
+			#else
+				localfilename = strrchr(localname, '/')+1;
+			#endif
 			items += 2;
 		}
 		if (need_preview_uri) {
@@ -460,7 +521,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	if (need_fifoin) items++;
 	if (need_fifoout) items++;
 	if (need_inplace) items++;
-	
+
 	table = g_new(Tconvert_table, items+1);
 	if (need_filename) {
 		table[cur].my_int = 'n';
@@ -545,6 +606,13 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 static gboolean outputbox_io_watch_lcb(GIOChannel *channel,GIOCondition condition,gpointer data) {
 	Texternalp *ep = data;
 	DEBUG_MSG("outputbox_io_watch_lcb, called with condition %d\n",condition);
+	if (!ep->bfwin->outputbox) {
+		DEBUG_MSG("outputbox_io_watch_lcb, outputbox == NULL, close the channel\n");
+		g_io_channel_shutdown(channel,TRUE,NULL);
+		externalp_unref(ep);
+		return FALSE;		
+	}
+	
 	if (condition & G_IO_IN) {
 		gchar *buf=NULL;
 		gsize buflen=0,termpos=0;
@@ -554,8 +622,8 @@ static gboolean outputbox_io_watch_lcb(GIOChannel *channel,GIOCondition conditio
 			if (buflen > 0) {
 				if (termpos < buflen) buf[termpos] = '\0';
 				fill_output_box(ep->bfwin->outputbox, buf);
-				g_free(buf);
 			}
+			g_free(buf);
 			status = g_io_channel_read_line(channel,&buf,&buflen,&termpos,&gerror);
 		}
 		if (gerror) {
@@ -612,10 +680,12 @@ static gboolean filter_io_watch_lcb(GIOChannel *channel,GIOCondition condition,g
 			gint end=ep->end;
 			GError *error=NULL;
 			DEBUG_MSG("filter_io_watch_lcb, received '%s'\n",str_return);
-			if (ep->end == -1) {
-				end = gtk_text_buffer_get_char_count(ep->bfwin->current_document->buffer);
+			if (ep->bfwin->current_document) {
+				if (ep->end == -1) {
+					end = gtk_text_buffer_get_char_count(ep->bfwin->current_document->buffer);
+				}
+				doc_replace_text(ep->bfwin->current_document, str_return, ep->begin, end);
 			}
-			doc_replace_text(ep->bfwin->current_document, str_return, ep->begin, end);
 			g_io_channel_shutdown(channel,TRUE,&error);
 			externalp_unref(ep);
 			g_free(str_return);
