@@ -2,7 +2,7 @@
  * html2.c - menu/toolbar callbacks, inserting functions, and other cool stuff 
  * otherwise html.c is getting so long ;-)
  *
- * Copyright (C) 1999-2009 Olivier Sessink
+ * Copyright (C) 1999-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 #include <stdlib.h>            /* strtol() */
 
 #include "htmlbar.h"
-#include "htmlbar_gui.h"
+#include "htmlbar_stock_icons.h"
 #include "html2.h"
 #include "cap.h"
 #include "../bf_lib.h"         /* string_is_color(), strip_any_whitespace()*/
@@ -138,7 +138,9 @@ typedef struct {
 	GtkWidget *win;
 	Tcs3_destination dest;
 	Tcs3_style styletype;
-	GtkWidget *clist;
+	/*GtkWidget *clist;*/
+	GtkListStore *lstore;
+	GtkWidget *lview; 
 	gint selected_row;
 	gboolean grab;
 	GtkWidget *selector;
@@ -325,7 +327,7 @@ static GList *pointer_arr2glist(gchar **arr) {
 
 static void cs3d_destroy_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 	window_destroy(diag->win);
-	g_free(diag);
+	g_slice_free(Tcs3_diag, diag);
 }
 
 static void cs3d_cancel_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
@@ -333,249 +335,153 @@ static void cs3d_cancel_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 }
 
 static void cs3d_ok_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
-
 	Tcs3_destination dest = diag->dest;
-	gchar *stylebuf=NULL;
+	GtkTreeIter iter;
+	GString *retstr;
+	retstr = g_string_new("");
 
 	if (diag->styletype == onestyle) {
-		gchar *tmpbuf1 = NULL;
-		gchar *tmpbuf2 = g_strdup("");
-		gint retval=1, row=0;
+		gboolean retval = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(diag->lstore), &iter);
 		while (retval) {
-			gchar *text[4];
-			retval = (gtk_clist_get_text(GTK_CLIST(diag->clist), row, 0, &text[0]) 
-						&& gtk_clist_get_text(GTK_CLIST(diag->clist), row, 1, &text[1]));
-			if (retval) {
-				tmpbuf1 = g_strconcat(tmpbuf2, text[0], ": ", text[1], "; ", NULL);
-				g_free(tmpbuf2);
-				tmpbuf2 = tmpbuf1;
-			}
-			row++;
+			gchar *text[3];
+			gtk_tree_model_get(GTK_TREE_MODEL(diag->lstore), &iter, 1, &text[1], 2, &text[2], -1);
+			retstr = g_string_append(retstr, text[1]);
+			retstr = g_string_append(retstr, ": ");
+			retstr = g_string_append(retstr, text[2]);
+			retstr = g_string_append(retstr, "; ");
+			retval = gtk_tree_model_iter_next(GTK_TREE_MODEL(diag->lstore), &iter);
 		}
-		stylebuf = tmpbuf2;
 	} else { /* multistyle */
-		gchar *tmpbuf1 = NULL;
-		gchar *tmpbuf2 = g_strdup("");
 		gchar *prev_selector = NULL;
-		gint retval=1, row=0;
+		gboolean retval = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(diag->lstore), &iter);
 		while (retval) {
-			gchar *text[4];
-			retval = (gtk_clist_get_text(GTK_CLIST(diag->clist), row, 0, &text[0])
-					&& gtk_clist_get_text(GTK_CLIST(diag->clist), row, 1, &text[1])
-					&& gtk_clist_get_text(GTK_CLIST(diag->clist), row, 2, &text[2]));
-			if (retval) {
-				if (!prev_selector) {
-					prev_selector = g_strdup(text[0]);
-					tmpbuf1 = g_strconcat(tmpbuf2, text[0], " {\n", NULL);
-					g_free(tmpbuf2);
-					tmpbuf2 = tmpbuf1;
-				} 
-				if (strcmp(prev_selector, text[0]) == 0 ) {
-					tmpbuf1 = g_strconcat(tmpbuf2, "\t", text[1], ": ", text[2], ";\n", NULL);
-				} else {
-					tmpbuf1 = g_strconcat(tmpbuf2, "}\n", text[0], " {\n\t", text[1], ": ", text[2], ";\n", NULL);
-					g_free(prev_selector);
-					prev_selector = g_strdup(text[0]);
-				}
-				g_free(tmpbuf2);
-				tmpbuf2 = tmpbuf1;
-				row++;
+			gchar *text[3];
+			gtk_tree_model_get(GTK_TREE_MODEL(diag->lstore), &iter, 0, &text[0], 1, &text[1], 2, &text[2], -1);
+			if (!prev_selector) {
+				prev_selector = g_strdup(text[0]);
+				retstr = g_string_append(retstr, text[0]);
+				retstr = g_string_append(retstr, " {\n");
+			} else if (g_strcmp0(prev_selector, text[0]) != 0 ) {
+				retstr = g_string_append(retstr, "}\n");
+				retstr = g_string_append(retstr, text[0]);
+				retstr = g_string_append(retstr, " {\n");
+				g_free(prev_selector);
+				prev_selector = g_strdup(text[0]);
 			}
+			retstr = g_string_append(retstr, "\t");
+			retstr = g_string_append(retstr, text[1]);
+			retstr = g_string_append(retstr, ": ");
+			retstr = g_string_append(retstr, text[2]);
+			retstr = g_string_append(retstr, ";\n");
+			retval = gtk_tree_model_iter_next(GTK_TREE_MODEL(diag->lstore), &iter);
 		}
 		g_free(prev_selector);
 		/* only append '}\n' in case there is content */
-		if (strlen(tmpbuf2) != 0)
-			stylebuf = g_strconcat(tmpbuf2, "}\n", NULL);
-		g_free(tmpbuf2);
+		if (retstr->len > 0) {
+			retstr = g_string_append(retstr, "}\n");
+		}
 	}
 
-	if (stylebuf) {
+	if (retstr->len > 0) {
 		if (dest.dest_type == entry) {
 			DEBUG_MSG("cs3d_ok_clicked_lcb, entry? %p\n", dest.entry);
 			if (dest.entry && GTK_IS_WIDGET(dest.entry)) {
 				DEBUG_MSG("cs3d_ok_clicked_lcb, entry!\n");
-				gtk_entry_set_text(GTK_ENTRY(dest.entry), stylebuf);
+				gtk_entry_set_text(GTK_ENTRY(dest.entry), retstr->str);
 			}
 		} else if (dest.dest_type == textbox) {
 			if (dest.doc) {
 				DEBUG_MSG("cs3d_ok_clicked_lcb, textbox!\n");
 				if (dest.doc_start != -1 || dest.doc_end != -1) {
-					doc_replace_text(dest.doc, stylebuf, dest.doc_start, dest.doc_end);
+					doc_replace_text(dest.doc, retstr->str, dest.doc_start, dest.doc_end);
 				} else {
-					doc_insert_two_strings(dest.doc, stylebuf, NULL);
+					doc_insert_two_strings(dest.doc, retstr->str, NULL);
 				}
 			}
 		} else if (dest.dest_type == wizard) {
 			GtkTextBuffer *buf;
 			buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(dest.entry));
-			gtk_text_buffer_set_text(buf, stylebuf, -1);
+			gtk_text_buffer_set_text(buf, retstr->str, -1);
 		} else {
 #ifdef DEVELOPMENT
 			g_print("cs3d_ok_clicked_lcb, an unknown dest type, this should never happen!\n");
 #endif
 		}
-		g_free(stylebuf);
 	}
+	g_string_free(retstr, TRUE);
 	cs3d_destroy_lcb(NULL, diag);
 }
 
-static void cs3d_select_row_lcb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, Tcs3_diag *diag) {
-	diag->selected_row = row;
-	if (diag->selected_row != -1) {
-		gchar *text[3];
-		if (diag->styletype == onestyle) {
-			text[0] = NULL;
-			gtk_clist_get_text(GTK_CLIST(diag->clist), diag->selected_row, 0, &text[1]);
-			gtk_clist_get_text(GTK_CLIST(diag->clist), diag->selected_row, 1, &text[2]);
-		} else { /* multistyle */
-			gtk_clist_get_text(GTK_CLIST(diag->clist), diag->selected_row, 0, &text[0]);
-			gtk_clist_get_text(GTK_CLIST(diag->clist), diag->selected_row, 1, &text[1]);
-			gtk_clist_get_text(GTK_CLIST(diag->clist), diag->selected_row, 2, &text[2]);
-		}
-		if (text[0]) {
-			gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->selector))), text[0]);
-		}
-		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->property))), text[1]);
-		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->value))), text[2]);
-		DEBUG_MSG("cs3d_select_row_lcb, sele=%s, prop=%s, val=%s\n", text[0], text[1], text[2]);
+static void cs3d_selection_changed_cb(GtkTreeSelection *selection, gpointer data) 
+{
+	Tcs3_diag *diag = data;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	gchar *text[3] = {NULL, NULL, NULL};
+	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+		gint *indices;
+		GtkTreePath *path;
+		gtk_tree_model_get(model, &iter, 0, &text[0], 1, &text[1], 2, &text[2], -1);
+		
+		/* now set the diag->selected_row !!!!!!!!!!!!!!!!!!! */
+		path = gtk_tree_model_get_path(model, &iter);
+		indices = gtk_tree_path_get_indices(path);
+		DEBUG_MSG("cs3d_selection_changed_cb, selected row=%d\n",indices[0]);
+		diag->selected_row = indices[0];
+		gtk_tree_path_free(path);
 	}
-}
-
-static void cs3d_unselect_row_lcb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, Tcs3_diag *diag) {
-	diag->selected_row = -1;
 	if (diag->styletype == multistyle) {
-		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->selector))), "");
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->selector))), text[0]?text[0]:"");
 	}
-	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->property))), "");
-	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->value))), "");
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->property))), text[1]?text[1]:"");
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->value))), text[2]?text[2]:"");
+	
+	
 }
 
 static void add_to_row(Tcs3_diag *diag, gint whichrow) {
-	gchar *text[4];
-	gint count, correct=1;
+	gchar *text[3] = {NULL, NULL, NULL};
+	gboolean correct=TRUE;
+	gint count=1;
 	if (diag->styletype == multistyle) {
-		DEBUG_MSG("add_to_row, multistyle\n");
 		text[0] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->selector))), 0, -1);
-		text[1] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->property))), 0, -1);
-		text[2] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->value))), 0, -1);
-	} else {
-		DEBUG_MSG("add_to_row, onestyle\n");
-		text[0] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->property))), 0, -1);
-		text[1] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->value))), 0, -1);
-		text[2] = NULL;
+		count=0;
 	}
-	text[3] = NULL;
+	text[1] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->property))), 0, -1);
+	text[2] = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->value))), 0, -1);
 
-	count=0;
-	while (text[count]) {
+	while (count < 3) {
 		if (strlen(text[count]) == 0) {
-			correct = 0;
+			DEBUG_MSG("add_to_row, %s:%s:%s: %d %s is incorrect, will not add\n",text[0],text[1], text[2], count, text[count]);
+			correct = FALSE;
+			break;
 		}
 		count++;
 	}
 	if (correct) {
+		GtkTreeIter iter;
 		if (whichrow == -1) {
-			gtk_clist_append(GTK_CLIST(diag->clist), text);
+			gtk_list_store_append(GTK_LIST_STORE(diag->lstore),&iter);
 		} else {
-			DEBUG_MSG("add_to_row, removing %d\n", whichrow);
-			gtk_clist_remove(GTK_CLIST(diag->clist), whichrow);
-			DEBUG_MSG("add_to_row, adding new %d\n", whichrow);
-			gtk_clist_insert(GTK_CLIST(diag->clist), whichrow, text);
+			gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(diag->lstore),&iter,NULL,whichrow);
 		}
-		gtk_clist_sort(GTK_CLIST(diag->clist));
+		gtk_list_store_set(GTK_LIST_STORE(diag->lstore), &iter, 0, text[0], 1, text[1], 2, text[2], -1);
 	}
-	count=0;
-	while (text[count]) {
-		g_free(text[count]);
-		count++;
-	}
-}
-
-static void cs3d_add_to_update(Tcs3_diag *diag, 
-                               const gchar *selector, const gchar *property, 
-                               const gchar *curValue, const gchar *newValue) {
-	/* _DM_ Alert box asking if user would like to change request from add to
-	 * update.
-	 */
-    const gchar *buttons[] = {GTK_STOCK_CANCEL, _("_Update"), NULL};
-    gchar *primaryText = NULL, *secondaryText = NULL; 
-    gint result;
-    
-    if (selector) {
-        primaryText = g_strdup_printf (_("The %s %s property already exists.\n"), selector, property);
-    } else {
-        primaryText = g_strdup_printf (_("The %s property already exists.\n"), property);
-    }
-    
-    secondaryText = g_strdup_printf (_("Update its value from %s to %s?"), curValue, newValue);
-    
-    result = message_dialog_new_multi(diag->win,
-                                      GTK_MESSAGE_QUESTION,
-                                      buttons,
-                                      primaryText,
-                                      secondaryText);
-
-	g_free (primaryText);
-	g_free (secondaryText);
-
-	if (result == 1) {
-	    add_to_row(diag, diag->selected_row);
-        gtk_clist_unselect_row(GTK_CLIST(diag->clist), diag->selected_row, 0);
-	}
-	
+	g_free(text[0]);
+	g_free(text[1]);
+	g_free(text[2]);
 }
 
 static void cs3d_add_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
-	gint onlist = 0, row = 0, retval = 1;
-	gint col = diag->styletype == onestyle ? 0 : 1;
-	gchar *cmb_selector = NULL, *cmb_property = NULL, *cmb_value = NULL;
-	gchar *lst_selector, *lst_property, *lst_value;
-
-	if (diag->styletype == multistyle) { 
-		cmb_selector = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->selector))), 0, -1);
-	}
-	cmb_property = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->property))), 0, -1);
-	cmb_value = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->value))), 0, -1);
-
-	while (retval) {
-		if (diag->styletype == multistyle) {
-			retval = gtk_clist_get_text(GTK_CLIST(diag->clist), row, 0, &lst_selector);
-			if (retval && g_strcmp0(lst_selector, cmb_selector) != 0) {
-				row++;
-				continue;
-			}
-		}
-		retval = gtk_clist_get_text(GTK_CLIST(diag->clist), row, col, &lst_property);
-		if (retval && strcmp(lst_property, cmb_property) == 0) {
-			retval = gtk_clist_get_text(GTK_CLIST(diag->clist), row, col + 1, &lst_value);
-			if (retval && strcmp(lst_value, cmb_value) == 0) {
-				onlist = 1;
-				DEBUG_MSG("%s already on list.\n", cmb_property);
-				break;
-			} else {
-				onlist = 1;
-				DEBUG_MSG("%s already on list.\n", cmb_property);
-				diag->selected_row = row;
-				cs3d_add_to_update(diag, cmb_selector, cmb_property, lst_value, cmb_value);
-				break;
-			}
-		}
-		DEBUG_MSG("clist %d contains: %s, %s\n", retval, lst_selector, lst_property); 
-		row++;
-	}
-    
-	if(!onlist)
-		add_to_row(diag, -1);
-
-	if (cmb_selector)
-		g_free (cmb_selector);
-	g_free (cmb_property);
-	g_free (cmb_value);
+	add_to_row(diag, -1);
 }
 
 static void cs3d_del_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 	if (diag->selected_row != -1) {
-		gtk_clist_remove(GTK_CLIST(diag->clist), diag->selected_row);
+		GtkTreeIter iter;
+		gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(diag->lstore),&iter,NULL,diag->selected_row); 
+		gtk_list_store_remove(GTK_LIST_STORE(diag->lstore),&iter);
+		/*gtk_clist_remove(GTK_CLIST(diag->clist), diag->selected_row);*/
 		diag->selected_row = -1;
 	}
 }
@@ -604,7 +510,7 @@ static void cs3d_prop_activate_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 			}
 			for (tmplist=g_list_first(list);tmplist;tmplist=g_list_next(tmplist)) {
 				if (tmplist->data) {
-					gtk_combo_box_append_text(GTK_COMBO_BOX(diag->value),tmplist->data);
+					gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(diag->value),tmplist->data);
 				}
 			}
 			g_list_free(list);
@@ -623,7 +529,7 @@ static void cs3d_prop_activate_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 		} else {
 				gtk_widget_set_sensitive(diag->extra_but, FALSE);
 		}
-		gtk_entry_set_editable(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(diag->value))), !tmp->force_pos);
+		gtk_editable_set_editable(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(diag->value))), !tmp->force_pos);
 	} else {
 		gtk_widget_set_sensitive(diag->extra_but, FALSE);
 	}
@@ -636,8 +542,11 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	GList *tmplist = NULL;
 	Tcs3_arr tmp;
 	gint count=0;
+	GtkTreeSelection *selection;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
 	
-	diag = g_malloc(sizeof(Tcs3_diag));
+	diag = g_slice_new(Tcs3_diag);
 	diag->win = window_full2(_("Cascading Style Sheet Builder"), GTK_WIN_POS_CENTER_ON_PARENT, 
 			12, G_CALLBACK(cs3d_destroy_lcb), diag, TRUE, transient_win);
 	gtk_window_set_role(GTK_WINDOW(diag->win), "css");
@@ -649,7 +558,7 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(diag->win),vbox);
 	
-	table = gtk_table_new(3, 6, FALSE);
+	table = gtk_table_new(3, 6, TRUE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 12);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
@@ -658,7 +567,7 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	if (diag->styletype == multistyle) {
 		tmplist = glist_with_html_tags(1);
 		diag->selector = combobox_with_popdown(NULL, tmplist,1);
-		bf_mnemonic_label_tad_with_alignment(_("_Selector:"), diag->selector, 0, 0.5, table, 0, 1, 0, 1);
+		dialog_mnemonic_label_in_table(_("_Selector:"), diag->selector, table, 0, 1, 0, 1);
 		gtk_table_attach_defaults(GTK_TABLE(table), diag->selector, 1 ,5 , 0, 1);
 		g_list_free(tmplist);
 		tmplist = NULL;
@@ -673,13 +582,13 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	diag->property = combobox_with_popdown(NULL, tmplist,1);
 	g_list_free(tmplist);
 	tmplist = NULL;
-	gtk_signal_connect(GTK_OBJECT(gtk_bin_get_child(GTK_BIN(diag->property))), "activate", G_CALLBACK(cs3d_prop_activate_lcb), diag);
-	gtk_signal_connect(GTK_OBJECT(gtk_bin_get_child(GTK_BIN(diag->property))), "changed", G_CALLBACK(cs3d_prop_activate_lcb), diag);
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(diag->property)), "activate", G_CALLBACK(cs3d_prop_activate_lcb), diag);
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(diag->property)), "changed", G_CALLBACK(cs3d_prop_activate_lcb), diag);
 
 	diag->value = combobox_with_popdown(NULL, tmplist,1);
-	bf_mnemonic_label_tad_with_alignment(_("_Property:"), diag->property, 0, 0.5, table, 0, 1, 1, 2);
+	dialog_mnemonic_label_in_table(_("_Property:"), diag->property, table, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(table), diag->property, 1, 5, 1, 2);
-	bf_mnemonic_label_tad_with_alignment(_("_Value:"), diag->value, 0, 0.5, table, 0, 1, 2, 3);
+	dialog_mnemonic_label_in_table(_("_Value:"), diag->value, table, 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(table), diag->value, 1, 5, 2, 3);
 	
 	gtk_widget_realize(diag->win);
@@ -689,14 +598,32 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 
 	/* the list widget and the buttons are in a horizontal box */
 	hbox = gtk_hbox_new(FALSE, 12);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 18);
-
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 18);
+	
+	diag->lstore = gtk_list_store_new(3, G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(diag->lstore),0,GTK_SORT_ASCENDING);
+	diag->lview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(diag->lstore));
+	g_object_unref(G_OBJECT(diag->lstore));
+	if (diag->styletype == multistyle) {
+		renderer = gtk_cell_renderer_text_new();
+		column = gtk_tree_view_column_new_with_attributes (_("Selector"),renderer,"text", 0,NULL);
+		gtk_tree_view_append_column(GTK_TREE_VIEW(diag->lview), column);
+	}
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes (_("Property"),renderer,"text", 1,NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(diag->lview), column);
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes (_("Value"),renderer,"text", 2,NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(diag->lview), column);
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(diag->lview));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(selection), "changed",G_CALLBACK(cs3d_selection_changed_cb),diag);
 	scrolwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_widget_set_size_request(scrolwin, 400, 300);
-	gtk_box_pack_start(GTK_BOX(hbox), scrolwin, FALSE, FALSE, 0);
-	
-	if (diag->styletype == multistyle) {
+	gtk_box_pack_start(GTK_BOX(hbox), scrolwin, TRUE, TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(scrolwin), diag->lview);
+	/*if (diag->styletype == multistyle) {
 		gchar *titles[] = {_("Selector"), _("Property"), _("Value"), NULL};
 		diag->clist = gtk_clist_new_with_titles(3, titles);
 	} else {
@@ -706,22 +633,24 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	gtk_clist_set_sort_column(GTK_CLIST(diag->clist), 0);
 	gtk_clist_set_auto_sort(GTK_CLIST(diag->clist), TRUE);
 
-/*	gtk_clist_columns_autosize(GTK_CLIST(diag->clist));*/
-	gtk_signal_connect(GTK_OBJECT(diag->clist), "select_row", GTK_SIGNAL_FUNC(cs3d_select_row_lcb), diag);
-	gtk_signal_connect(GTK_OBJECT(diag->clist), "unselect_row", GTK_SIGNAL_FUNC(cs3d_unselect_row_lcb), diag);
-
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolwin), diag->clist);
-
+	g_signal_connect(diag->clist, "select_row", G_CALLBACK(cs3d_select_row_lcb), diag);
+	g_signal_connect(diag->clist, "unselect_row", G_CALLBACK(cs3d_unselect_row_lcb), diag);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolwin), diag->lview);
+	*/
+	
 	vbox2 = gtk_vbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);
 	
-	but = bf_generic_mnemonic_button(_(" _Add "), G_CALLBACK(cs3d_add_clicked_lcb), diag);
+	but = gtk_button_new_with_mnemonic(_(" _Add "));
+	g_signal_connect(but, "clicked", G_CALLBACK(cs3d_add_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
 
-	but = bf_generic_mnemonic_button(_(" _Update "), G_CALLBACK(cs3d_update_clicked_lcb), diag);
+	but = gtk_button_new_with_mnemonic(_(" _Update "));
+	g_signal_connect(but, "clicked", G_CALLBACK(cs3d_update_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
 	
-	but = bf_generic_mnemonic_button(_(" _Delete "), G_CALLBACK(cs3d_del_clicked_lcb), diag);
+	but = gtk_button_new_with_mnemonic(_(" _Delete "));
+	g_signal_connect(but, "clicked", G_CALLBACK(cs3d_del_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
 
 	/* the ok and cancel button are in a horizontal box below */
@@ -730,8 +659,8 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);	
 
 	hbox = gtk_hbutton_box_new();
-	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 12);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_box_set_spacing(GTK_BOX(hbox), 12);
 	
 	but = bf_stock_cancel_button(G_CALLBACK(cs3d_cancel_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, FALSE, 0);
@@ -770,10 +699,9 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 	if (diag->styletype == onestyle) {
 		gint prev_end = 0;
 		Tcs3_prevtype_onestyle prevtype = end;
-		gchar *text[3];
+		gchar *text[3] = {NULL, NULL, NULL};
 		
 		DEBUG_MSG("css_parse, onestyle\n");
-		text[2] = NULL;
 		while (tmp[count] && parsable_css) {
 			switch (tmp[count]) {
 			case ':':
@@ -819,7 +747,10 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 			break;
 			}
 			if (prevtype == value) {
-				gtk_clist_append(GTK_CLIST(diag->clist), text);
+				GtkTreeIter iter;
+				gtk_list_store_append(diag->lstore, &iter);
+				gtk_list_store_set(diag->lstore,&iter,1, text[0],2,text[1],-1);
+				/*gtk_clist_append(GTK_CLIST(diag->clist), text);*/
 				g_free(text[0]);
 				g_free(text[1]);
 				prevtype = end;
@@ -895,7 +826,11 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 			break;
 			}
 			if (prevtype == mvalue) {
-				gtk_clist_append(GTK_CLIST(diag->clist), text);
+				GtkTreeIter iter;
+				gtk_list_store_append(diag->lstore, &iter);
+				gtk_list_store_set(diag->lstore,&iter,0,text[0],1, text[1],2,text[2],-1);
+
+				/*gtk_clist_append(GTK_CLIST(diag->clist), text);*/
 				g_free(text[1]);
 				text[1] = NULL;
 				g_free(text[2]);
@@ -910,7 +845,7 @@ static void css_parse(Tcs3_diag *diag, gchar *data) {
 			}
 		}
 	}
-	gtk_clist_sort(GTK_CLIST(diag->clist));
+	/*gtk_clist_sort(GTK_CLIST(diag->clist));*/
 	DEBUG_MSG("css_parse, finished\n");
 }
 
@@ -973,10 +908,10 @@ GtkWidget *style_but_new(GtkWidget * which_entry, GtkWidget * win)
 
 	style_but = gtk_button_new();
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), htmlbar_pixmap(pixmap_css_small),FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_image_new_from_stock(BF_STOCK_CSS_SMALL, GTK_ICON_SIZE_BUTTON),FALSE, FALSE, 3);
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new_with_mnemonic(_("_Style...")), TRUE, TRUE, 3);
 	gtk_container_add(GTK_CONTAINER(style_but), hbox);
-	gtk_signal_connect(GTK_OBJECT(style_but), "clicked", G_CALLBACK(style_but_clicked_lcb), which_entry);
+	g_signal_connect(style_but, "clicked", G_CALLBACK(style_but_clicked_lcb), which_entry);
 	gtk_widget_show_all(style_but);
 	return style_but;
 }
@@ -1010,10 +945,10 @@ GtkWidget *style_but_new_for_wizard(GtkWidget * textview) {
 
 	style_but = gtk_button_new();
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), htmlbar_pixmap(pixmap_css_small),FALSE, FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_image_new_from_stock(BF_STOCK_CSS_SMALL, GTK_ICON_SIZE_BUTTON),FALSE, FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new_with_mnemonic(_("_Style...")), TRUE, TRUE, 6);
 	gtk_container_add(GTK_CONTAINER(style_but), hbox);
-	gtk_signal_connect(GTK_OBJECT(style_but), "clicked", G_CALLBACK(style_but_for_wizard_clicked_lcb), textview);
+	g_signal_connect(style_but, "clicked", G_CALLBACK(style_but_for_wizard_clicked_lcb), textview);
 	gtk_widget_show_all(style_but);
 	return style_but;
 }
@@ -1101,7 +1036,6 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	gtk_container_add(GTK_CONTAINER(csd->win), vbox);
 	csd->csel = gtk_color_selection_new();
 	gtk_color_selection_set_has_opacity_control(GTK_COLOR_SELECTION(csd->csel), FALSE);
-	
 	if (this_color) {
 		if (gdk_color_parse(this_color, &gcolor)) {
 			gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(csd->csel), &gcolor);
@@ -1109,20 +1043,6 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 			this_color=NULL;
 		}
 	}
-	if (bfwin && bfwin->session->colorlist) {
-		GtkSettings* gtksettings;
-		gchar *strings = stringlist_to_string(bfwin->session->colorlist, ":");
-		strings[strlen(strings)-1] = '\0';
-		gtksettings = gtk_widget_get_settings(GTK_WIDGET(csd->csel));
-		g_object_set(G_OBJECT(gtksettings), "gtk-color-palette", strings, NULL); 
-/*		DEBUG_MSG("colsel_dialog, setting palette from %s\n", strings);
-		if (gtk_color_selection_palette_from_string(strings, &gcolorarr, &num)) {
-			DEBUG_MSG("num=%d, gcolorarr=%p\n",num,gcolorarr);
-		} else {
-			DEBUG_MSG("hmm, failed to parse our string :(\n");
-		} */
-		g_free(strings);
-	}
 	gtk_color_selection_set_has_palette(GTK_COLOR_SELECTION(csd->csel), TRUE);
 	gtk_box_pack_start(GTK_BOX(vbox), csd->csel, TRUE, TRUE, 0);
 
@@ -1131,8 +1051,8 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);
 
 	hbox = gtk_hbutton_box_new();
-	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 12);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_box_set_spacing(GTK_BOX(hbox), 12);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	but = bf_stock_cancel_button(G_CALLBACK(colsel_cancel_clicked_lcb), csd);
@@ -1140,6 +1060,30 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	but = bf_stock_ok_button(G_CALLBACK(colsel_ok_clicked_lcb), csd);
 	gtk_window_set_default(GTK_WINDOW(csd->win), but);
 	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
+
+	if (bfwin && bfwin->session->colorlist) {
+		GtkSettings* gtksettings;
+		/* Note that this function can only be called when the GtkWidget is attached to a toplevel, since the settings object is specific to a particular GdkScreen.  */
+		gtksettings = gtk_widget_get_settings(GTK_WIDGET(csd->csel));
+		if (gtksettings) {
+			gchar *strings;
+			DEBUG_MSG("pallette list=%d\n",g_list_length(bfwin->session->colorlist));
+			bfwin->session->colorlist = limit_stringlist(bfwin->session->colorlist, 20, TRUE);
+			DEBUG_MSG("pallette list=%d\n",g_list_length(bfwin->session->colorlist));
+			strings = stringlist_to_string(bfwin->session->colorlist, ":");
+			strings[strlen(strings)-1] = '\0';
+			/* this property may contain max 20 colors, otherwise gtk will crash */
+			g_object_set(G_OBJECT(gtksettings), "gtk-color-palette", strings, NULL);
+			g_free(strings);
+		} 
+/*		DEBUG_MSG("colsel_dialog, setting palette from %s\n", strings);
+		if (gtk_color_selection_palette_from_string(strings, &gcolorarr, &num)) {
+			DEBUG_MSG("num=%d, gcolorarr=%p\n",num,gcolorarr);
+		} else {
+			DEBUG_MSG("hmm, failed to parse our string :(\n");
+		} */
+	}
+
 	gtk_widget_show_all(csd->win);
 
 	DEBUG_MSG("colsel_dialog, finished\n");
@@ -1216,7 +1160,7 @@ GtkWidget *color_but_new(GtkWidget * which_entry, GtkWidget * win)
 	GtkWidget *color_but;
 
 	color_but = gtk_button_new_from_stock(GTK_STOCK_SELECT_COLOR);
-	gtk_signal_connect(GTK_OBJECT(color_but), "clicked", G_CALLBACK(color_but_clicked), which_entry);
+	g_signal_connect(color_but, "clicked", G_CALLBACK(color_but_clicked), which_entry);
 	gtk_widget_show(color_but);
 	return color_but;
 }
