@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * quickstart.c - quickstart dialog
  *
- * Copyright (C) 2005-2010 James Hayward and Olivier Sessink
+ * Copyright (C) 2005-2011 James Hayward and Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 /* indented with indent -ts4 -kr -l110   */
 
@@ -27,10 +26,10 @@
 #include "htmlbar.h"
 #include "cap.h"
 #include "../config.h"
+#include "../bfwin.h"
 #include "../dialog_utils.h"
 #include "../document.h"
 #include "../gtk_easy.h"
-#include "../gui.h"
 #include "../stringlist.h"
 #include "quickstart.h"
 
@@ -181,7 +180,7 @@ quickstart_load_metatags(GtkListStore * lstore)
 		"http-equiv=\"content-type\" content=\"application/xhtml+xml; charset=UTF-8\"",
 		"http-equiv=\"content-style-type\" content=\"text/css\"",
 		"http-equiv=\"expires\" content=\"0\"",
-		"http-equiv=\"refresh\" content=\"5; URL=http://\"",
+		/*"http-equiv=\"refresh\" content=\"5; URL=http://\"",*/
 	};
 
 	for (i = 0; i < G_N_ELEMENTS(metaTags); i++) {
@@ -347,7 +346,7 @@ quickstart_response_lcb(GtkDialog * dialog, gint response, TQuickStart * qstart)
 			model = gtk_combo_box_get_model(GTK_COMBO_BOX(qstart->stylelinktype));
 			gtk_tree_model_get(model, &iter, 0, &name, -1);
 
-			stylehref = gtk_editable_get_chars(GTK_EDITABLE(GTK_BIN(qstart->stylehref)->child), 0, -1);
+			stylehref = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(qstart->stylehref))), 0, -1);
 			qstart->bfwin->session->urllist = add_to_stringlist(qstart->bfwin->session->urllist, stylehref);
 			stylemedia = gtk_editable_get_chars(GTK_EDITABLE(qstart->stylemedia), 0, -1);
 			styletitle = gtk_editable_get_chars(GTK_EDITABLE(qstart->styletitle), 0, -1);
@@ -394,7 +393,7 @@ quickstart_response_lcb(GtkDialog * dialog, gint response, TQuickStart * qstart)
 			stylearea = g_strdup("");
 		}
 
-		tmpstr2 = gtk_editable_get_chars(GTK_EDITABLE(GTK_BIN(qstart->scriptsrc)->child), 0, -1);
+		tmpstr2 = gtk_editable_get_chars(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(qstart->scriptsrc))), 0, -1);
 		if (strlen(tmpstr2) > 0) {
 			scriptsrc =
 				g_strconcat("<script type=\"text/javascript\" src=\"", tmpstr2, "\"></script>\n", NULL);
@@ -428,7 +427,7 @@ quickstart_response_lcb(GtkDialog * dialog, gint response, TQuickStart * qstart)
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(qstart->openNewDoc))) {
 			Tdocument *doc;
 			doc = doc_new(qstart->bfwin, FALSE);
-			switch_to_document_by_pointer(qstart->bfwin, doc);
+			bfwin_switch_to_document_by_pointer(qstart->bfwin, doc);
 		}
 
 		doc_insert_two_strings(qstart->bfwin->current_document,
@@ -496,9 +495,6 @@ static GtkWidget *
 quickstart_style_page_create(TQuickStart * qstart)
 {
 	GtkWidget *frame, *vbox, *vbox2, *hbox, *table, *label;
-	GtkListStore *history;
-	GList *urllist;
-	GtkTreeIter iter;
 	unsigned int i = 0;
 
 	const gchar *type[] = {
@@ -519,9 +515,9 @@ quickstart_style_page_create(TQuickStart * qstart)
 	hbox = gtk_hbox_new(FALSE, 12);
 	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
 
-	qstart->stylelinktype = gtk_combo_box_new_text();
+	qstart->stylelinktype = gtk_combo_box_text_new();
 	for (i = 0; i < G_N_ELEMENTS(type); i++) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(qstart->stylelinktype), type[i]);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(qstart->stylelinktype), type[i]);
 	}
 	g_signal_connect(G_OBJECT(qstart->stylelinktype), "changed", G_CALLBACK(quickstart_stylelinktype_changed),
 					 qstart);
@@ -533,16 +529,16 @@ quickstart_style_page_create(TQuickStart * qstart)
 
 	table = dialog_table_in_vbox(3, 2, 0, vbox2, FALSE, FALSE, 0);
 
-	history = gtk_list_store_new(1, G_TYPE_STRING);
+	/*history = gtk_list_store_new(1, G_TYPE_STRING);
 	urllist = g_list_first(qstart->bfwin->session->urllist);
 	while (urllist) {
 		gtk_list_store_append(history, &iter);
 		gtk_list_store_set(history, &iter, 0, urllist->data, -1);
 		urllist = g_list_next(urllist);
 	}
-
 	qstart->stylehref = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(history), 0);
-	g_object_unref(history);
+	g_object_unref(history);*/
+	qstart->stylehref = combobox_with_popdown(NULL, qstart->bfwin->session->urllist, TRUE);
 	dialog_mnemonic_label_in_table(_("HRE_F:"), qstart->stylehref, table, 0, 1, 0, 1);
 	gtk_table_attach(GTK_TABLE(table), qstart->stylehref, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0,
 					 0);
@@ -576,9 +572,6 @@ static GtkWidget *
 quickstart_script_page_create(TQuickStart * qstart)
 {
 	GtkWidget *frame, *vbox, *vbox2, *hbox, *label;
-	GtkListStore *history;
-	GList *urllist;
-	GtkTreeIter iter;
 
 	frame = gtk_frame_new(NULL);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
@@ -591,16 +584,16 @@ quickstart_script_page_create(TQuickStart * qstart)
 	hbox = gtk_hbox_new(FALSE, 12);
 	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
 
-	history = gtk_list_store_new(1, G_TYPE_STRING);
+	/*history = gtk_list_store_new(1, G_TYPE_STRING);
 	urllist = g_list_first(qstart->bfwin->session->urllist);
 	while (urllist) {
 		gtk_list_store_append(history, &iter);
 		gtk_list_store_set(history, &iter, 0, urllist->data, -1);
 		urllist = g_list_next(urllist);
 	}
-
 	qstart->scriptsrc = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(history), 0);
-	g_object_unref(history);
+	g_object_unref(history);*/
+	qstart->scriptsrc = combobox_with_popdown(NULL,qstart->bfwin->session->urllist , TRUE);
 	label = gtk_label_new_with_mnemonic(_("_Src:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), qstart->scriptsrc);
@@ -622,7 +615,7 @@ void
 quickstart_dialog_new(Tbfwin * bfwin)
 {
 	TQuickStart *qstart;
-	GtkWidget *dialog, *table, *frame, *page, *alignment;
+	GtkWidget *dialog, *table, *frame, *page, *alignment, *vbox;
 	GtkListStore *headStore;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -643,14 +636,17 @@ quickstart_dialog_new(Tbfwin * bfwin)
 										 GTK_DIALOG_DESTROY_WITH_PARENT,
 										 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 										 GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
+#endif /* gtk3 */
+	vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(quickstart_response_lcb), qstart);
 
-	table = dialog_table_in_vbox_defaults(4, 3, 6, GTK_DIALOG(dialog)->vbox);
+	table = dialog_table_in_vbox_defaults(4, 3, 6, vbox);
 
-	qstart->dtd = gtk_combo_box_new_text();
+	qstart->dtd = gtk_combo_box_text_new();
 	for (i = 0; i < G_N_ELEMENTS(dtds); i++) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(qstart->dtd), dtds[i].name);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(qstart->dtd), dtds[i].name);
 	}
 	gtk_combo_box_set_active(GTK_COMBO_BOX(qstart->dtd), 0);
 	dialog_mnemonic_label_in_table(_("_DTD:"), qstart->dtd, table, 0, 1, 0, 1);
@@ -705,7 +701,7 @@ quickstart_dialog_new(Tbfwin * bfwin)
 	alignment = gtk_alignment_new(0, 0.5, 0, 0);
 	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 6, 8, 4, 0);
 	gtk_container_add(GTK_CONTAINER(alignment), qstart->openNewDoc);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), alignment, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, FALSE, 0);
 
 	gtk_widget_show_all(dialog);
 }
